% Copiright (c) 2016 - A. Panichella

%% prepare training set
global train_metrics train_oracle;
load(train_file);
training=table;
train_metrics=[training.wmc, training.dit, training.noc, training.cbo, training.rfc, training.lcom, training.loc];
train_oracle=training.bug;

%% prepare test set
global test_metrics test_oracle;
load(test_file);
test=table;
test_metrics=[test.wmc, test.dit, test.noc, test.cbo, test.rfc, test.lcom, test.loc];
test_oracle=test.bug;

%% Generalized Linear Regression
glm = glmfit(train_metrics,train_oracle,'poisson','link','identity');
[auc, graph] = evaluateGLM(glm);
result.glm.auc = auc;
result.glm.graph = graph;
hold off
plot(result.glm.graph.x, result.glm.graph.y,'b');
legend('GLM','Location','southeast');
hold on
pause(0.1)

%% Plot Optimal Curve
[auc, graph] = optimal_curve(test_oracle, test_metrics);
result.optimal.auc = auc;
result.optimal.graph = graph;
plot(result.optimal.graph.x, result.optimal.graph.y,'g');
legend('GLM','Optimal Curve','Location','southeast');
pause(0.1)

%% Run GA
start=cputime();
tot_runs=10;
nvar=8;%sum(~isnan(regression_tree.CutPoint)); %number of decision nodes in ctree
for nrun=1:tot_runs
    fprintf('Number of runs %8.3f  \n',nrun)
    population = [];
    % Start with the default options
    options = gaoptimset;
    ptions = gaoptimset(options,'PopulationSize', 200);
    options = gaoptimset(options,'EliteCount', 5);
    options = gaoptimset(options,'StallGenLimit', 100);
    options = gaoptimset(options,'TolFun', 0);
    options = gaoptimset(options,'TolCon', 0);
    options = gaoptimset(options,'InitialPopulation', population);
    options = gaoptimset(options,'SelectionFcn', {@selectionroulette});
    options = gaoptimset(options,'CrossoverFcn', @BLX);
    options = gaoptimset(options,'MutationFcn', {@mutationpolynomial});
    options = gaoptimset(options,'Display', 'iter');
    options = gaoptimset(options,'Generations',400);
    [x,fval,exitflag,output,population,score] = ga(@fitnessGLM,nvar,[],[],[],[],[],[],[],[],options);
   
    result.solutions(nrun,:)=x;
    result.ga.fval(nrun,:)=fval;
    [auc, graph]=evaluateGLM(result.solutions(nrun,:)');
    result.ga.auc(nrun)=auc;
    result.ga.graph(nrun)=graph;
    %plot
    plot(result.ga.graph(nrun).x, result.ga.graph(nrun).y,'r');
    legend('GLM','Optimal Curve','GA-GLM','Location','southeast');
    pause(0.1)
end

stop=cputime();
%% update AUC according to the optimal curve
result.ga.auc = result.ga.auc/result.optimal.auc;
result.glm.auc = result.glm.auc/result.optimal.auc;
fprintf('Delta_{effort} for traditional GLM  %8.3f  \n',mean(result.glm.auc))
fprintf('Delta_{effort} for GA-GLM %8.3f  \n',mean(result.ga.auc))
save(result_file,'-struct','result');
exectime(length(exectime)+1)=stop-start;